@@ -3,12 +3,29 @@ package POE::Component::Server::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.350'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
 use Errno qw(ECONNABORTED ECONNRESET);
 
+BEGIN {
+  # under perl-5.6.2 the warning "leaks" from the eval, while newer versions don't...
+  # it's due to Exporter.pm behaving differently, so we have to shut it up
+  no warnings 'redefine';
+  local *Carp::carp = sub { die @_ };
+
+  # Socket::GetAddrInfo provides getaddrinfo where earlier Perls' Socket don't.
+  eval { Socket->import('getaddrinfo') };
+  if ($@) {
+    # :newapi is legacy, but we include it to be sure in case the user has an old version of GAI
+    eval { require Socket::GetAddrInfo; Socket::GetAddrInfo->import( qw(:newapi getaddrinfo) ) };
+    if ($@) {
+      *getaddrinfo = sub { Carp::confess("Unable to use IPv6: Socket::GetAddrInfo not available") };
+    }
+  }
+}
+
 # Explicit use to import the parameter constants.
 use POE::Session;
 use POE::Driver::SysRW;
@@ -117,6 +134,16 @@ sub new {
 
   my $client_args = delete($param{ClientArgs}) || delete($param{Args});
 
+  if ( (defined $client_infilter and ! defined $client_outfilter) or
+    (defined $client_outfilter and ! defined $client_infilter) ) {
+    croak "ClientInputFilter must be used with ClientOutputFilter";
+  }
+
+  if (defined $client_filter and defined $client_infilter) {
+    carp "ClientFilter ignored with ClientInputFilter and ClientOutputFilter";
+    undef $client_filter;
+  }
+
   # Defaults.
 
   $concurrency = -1 unless defined $concurrency;
@@ -264,8 +291,7 @@ sub new {
                 $heap->{remote_ip} = inet_ntoa($remote_addr);
               }
               else {
-                $heap->{remote_ip} =
-                  Socket6::inet_ntop($domain, $remote_addr);
+                $heap->{remote_ip} = ( getaddrinfo($remote_addr) )[1];
               }
 
               $heap->{remote_port} = $remote_port;
@@ -533,14 +559,9 @@ sub _get_filters {
         "InputFilter"  => _load_filter($client_infilter),
         "OutputFilter" => _load_filter($client_outfilter)
       );
-      if (defined $client_filter) {
-        carp(
-          "ClientFilter ignored with ClientInputFilter or ClientOutputFilter"
-        );
-      }
     }
     elsif (defined $client_filter) {
-     return ( "Filter" => _load_filter($client_filter) );
+      return ( "Filter" => _load_filter($client_filter) );
     }
     else {
       return ( Filter => POE::Filter::Line->new(), );
@@ -706,8 +727,8 @@ POE::Session's _start handlers.
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
 component's default acceptor has trouble accepting a connection.
-C<Error> receives the usual ones for POE::Wheel::SocketFactory and
-POE::Wheel::ReadWrite L<ErrorEvent>.
+C<Error> receives the usual ones for L<POE::Wheel::SocketFactory/FailureEvent> and
+L<POE::Wheel::ReadWrite/ErrorEvent>.
 
 =head2 Default Child Connection Sessions
 
@@ -723,8 +744,8 @@ connection---and not in the master listening session.  This has been a
 major point of confusion.  We welcome suggestions for making this
 clearer.
 
-TODO - Document some of the implications of having each connection
-handled by a separate session.
+Z<TODO - Document some of the implications of having each connection
+handled by a separate session.>
 
 The component's C<ClientInput> callback defines how child sessions
 will handle input from their clients.  Its parameters are that of
@@ -822,12 +843,12 @@ session that will be listening for new connections.
 Many of the constructor parameters have been previously described.
 They are covered briefly again below.
 
-=head3 Server Sesson Configuration
+=head3 Server Session Configuration
 
 These constructor parameters affect POE::Component::Server::TCP's main
 listening session.
 
-TODO - Document the shutdown procedure somewhere.
+Z<TODO - Document the shutdown procedure somewhere.>
 
 =head4 Acceptor
 
@@ -838,7 +859,7 @@ C<SuccessEvent>.
 
 C<Acceptor> is optional; the default handler will create a new session
 for each connection.  All the "Client" constructor parameters are used
-to customize this session.  In other words, C<CleintInput> and such
+to customize this session.  In other words, C<ClientInput> and such
 B<are not used when C<Acceptor> is set>.
 
 The default C<Acceptor> adds significant convenience and flexibility
@@ -864,7 +885,7 @@ supports.  At the time of this writing, that may be a dotted IPv4
 quad, an IPv6 address, a host name, or a packed Internet address.  See
 also L</Hostname>.
 
-TODO - Example, using the lines below.
+Z<TODO - Example, using the lines below.>
 
   Address => '127.0.0.1'   # Localhost IPv4
   Address => "::1"         # Localhost IPv6
@@ -894,7 +915,7 @@ connections.  This may be useful if a server must perform lengthy
 initialization before allowing connections.  When the initialization
 finishes, it can yield(set_concurrency => -1) to enable connections.
 Likewise, a running server may yield(set_concurrency => 0) or any
-other number to dynamically tune its concurrency.  See L<EVENTS> for
+other number to dynamically tune its concurrency.  See L</EVENTS> for
 more about the set_concurrency event.
 
 Note: For C<Concurrency> to work with a custom C<Acceptor>, the
@@ -931,8 +952,10 @@ C<Domain> sets the address or protocol family within which to operate.
 The C<Domain> may be any value that POE::Wheel::SocketFactory
 supports.  AF_INET (Internet address space) is used by default.
 
-Use AF_INET6 for IPv6 support.  This constant is exported by Socket6,
-which must be loaded B<before> POE::Component::Server::TCP.
+Use AF_INET6 for IPv6 support.  This constant is exported by L<Socket>
+or L<Socket6>, depending on your version of Perl. Also be sure to have
+L<Socket::GetAddrInfo> installed, which is required by
+L<POE::Wheel::SocketFactory> for IPv6 support.
 
 =head4 Error
 
@@ -1121,7 +1144,9 @@ given object.
   ClientFilter => POE::Filter::Line->new(Literal => "\n"),
 
 C<ClientFilter> is optional.  The component will use
-"POE::Filter::Line" if it is omitted.
+"POE::Filter::Line" if it is omitted.  There is L</ClientInputFilter>
+and L</ClientOutputFilter> if you want to specify a different filter
+for both directions.
 
 Filter modules are not automatically loaded.  Be sure that the program
 loads the class before using it.
@@ -1172,7 +1197,8 @@ prohibits the other.
 
 C<ClientInputFilter> is used with C<ClientOutputFilter> to specify
 different protocols for input and output.  Both must be used together.
-Both follow the same usage as L</ClientFilter>.
+Both follow the same usage as L</ClientFilter>.  Overrides the filter set
+by L</ClientFilter>.
 
   ClientInputFilter  => [ "POE::Filter::Line", Literal => "\n" ],
   ClientOutputFilter => 'POE::Filter::Stream',
@@ -1181,7 +1207,8 @@ Both follow the same usage as L</ClientFilter>.
 
 C<ClientOutputFilter> is used with C<ClientInputFilter> to specify
 different protocols for input and output.  Both must be used together.
-Both follow the same usage as L</ClientFilter>.
+Both follow the same usage as L</ClientFilter>.  Overrides the filter set
+by L</ClientFilter>.
 
   ClientInputFilter  => POE::Filter::Line->new(Literal => "\n"),
   ClientOutputFilter => 'POE::Filter::Stream',
@@ -1238,7 +1265,7 @@ in its L<Alias> parameter.
 =head3 disconnected
 
 The "disconnected" event informs the TCP server that a connection was
-closed.  It is needed when using L<Concurrency> with an L<Acceptor>
+closed.  It is needed when using L</Concurrency> with an L</Acceptor>
 callback.  The custom Acceptor must provide its own disconnect
 notification so that the server's connection counting logic works.
 
@@ -1248,7 +1275,7 @@ server will never know when clients have disconnected.
 =head3 set_concurrency
 
 "set_concurrency" set the number of simultaneous connections the
-server will be willing to accept.  See L<Concurrency> for more
+server will be willing to accept.  See L</Concurrency> for more
 details.  "set_concurrency" must have one parameter: the new maximum
 connection count.
 
@@ -1383,7 +1410,7 @@ Some use cases require different session classes for the listener and
 the connection handlers.  This isn't currently supported.  Please send
 patches. :)
 
-TODO - Document that Reuse is set implicitly.
+Z<TODO - Document that Reuse is set implicitly.>
 
 =head1 AUTHORS & COPYRIGHTS
 
