@@ -3,7 +3,7 @@ package POE::Session;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.350'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Errno;
@@ -11,6 +11,7 @@ use Errno;
 sub SE_NAMESPACE    () { 0 }
 sub SE_OPTIONS      () { 1 }
 sub SE_STATES       () { 2 }
+sub SE_ID           () { 3 }
 
 sub CREATE_ARGS     () { 'args' }
 sub CREATE_OPTIONS  () { 'options' }
@@ -426,7 +427,7 @@ sub _invoke_state {
       $! = exists &Errno::ENOSYS ? &Errno::ENOSYS : &Errno::EIO;
       if ($self->[SE_OPTIONS]->{+OPT_DEFAULT} and $state ne EN_SIGNAL) {
         my $loggable_self =
-          $POE::Kernel::poe_kernel->_data_alias_loggable($self);
+          $POE::Kernel::poe_kernel->_data_alias_loggable($self->ID);
         POE::Kernel::_warn(
           "a '$state' event was sent from $file at $line to $loggable_self ",
           "but $loggable_self has neither a handler for it ",
@@ -589,8 +590,13 @@ sub _register_state {
 # Return the session's ID.  This is a thunk into POE::Kernel, where
 # the session ID really lies.
 
+sub _set_id {
+  my ($self, $id) = @_;
+  $self->[SE_ID] = $id;
+}
+
 sub ID {
-  $POE::Kernel::poe_kernel->ID_session_to_id(shift);
+  return shift()->[SE_ID];
 }
 
 #------------------------------------------------------------------------------
@@ -813,7 +819,7 @@ dispatch interface to those handlers.
 Consider the L</SYNOPSIS> for example.  A POE::Session instance is
 created with two C<inline_states>, each mapping an event name
 ("_start" and "next") to an inline subroutine.  POE::Session ensures
-that C<$_[KERNEL]> and so on are meaningful within an event handler.
+that L</$_[KERNEL]> and so on are meaningful within an event handler.
 
 Event handlers may also be object or class methods, using
 L</object_states> and L</package_states> respectively.  The create()
@@ -829,7 +835,7 @@ not usually necessary.  If an application is not careful about
 cleaning up these references you will create circular references,
 which will leak memory when POE::Kernel would normally destroy the
 POE::Session object.  It is recommended that you keep the session's
-C</ID> instead.
+L<ID> instead.
 
 =head2 POE::Session's Calling Convention
 
@@ -841,7 +847,7 @@ POE::Session's unconventional calling convention.  For example:
     ...;
   }
 
-Or the use of C</$_[KERNEL]>, C</$_[HEAP]> and C</$_[ARG0]> inline,
+Or the use of C<$_[KERNEL]>, C<$_[HEAP]> and C<$_[ARG0]> inline,
 as is done in most examples.
 
 What's going on here is rather basic.  Perl passes parameters into
@@ -849,7 +855,7 @@ subroutines or methods using the @_ array.  C<KERNEL>, C<HEAP>,
 C<ARG0> and others are constants exported by POE::Session (which is
 included for free when a program uses POE).
 
-So C</$_[KERNEL]> is an event handler's KERNELth parameter.
+So C<$_[KERNEL]> is an event handler's KERNELth parameter.
 C<@_[HEAP, ARG0]> is a slice of @_ containing the HEAPth and ARG0th
 parameters.
 
@@ -921,11 +927,11 @@ POE's parameter offsets:
 
 C<$_[SESSION]> is a reference to the current session object.  This lets event
 handlers access their session's methods.  Programs may also compare
-C<$_[SESSION]> to C</$_[SENDER]> to verify that intra-session events did not
+C<$_[SESSION]> to C<$_[SENDER]> to verify that intra-session events did not
 come from other sessions.
 
 C<$_[SESSION]> may also be used as the destination for intra-session
-C<post()|POE::Kernel/post> and C<call()|POE::Kernel/call>.  C<yield()|POE::Kernel/yield> is marginally more convenient and
+L<post()|POE::Kernel/post> and L<call()|POE::Kernel/call>.  L<yield()|POE::Kernel/yield> is marginally more convenient and
 efficient than C<post($_[SESSION], ...)> however.
 
 It is bad form to access another session directly.  The recommended
@@ -1022,8 +1028,8 @@ current ones.
     $_[KERNEL]->post( $_[SENDER], "echo", @_[ARG0..$#_] );
   }
 
-TODO - Document which events should have $_[SENDER] == $_[KERNEL].
-Probably in POE::Kernel.
+Z<TODO - Document which events should have $_[SENDER] == $_[KERNEL].
+Probably in POE::Kernel.>
 
 =head3 $_[CALLER_FILE], $_[CALLER_LINE] and $_[CALLER_STATE]
 
@@ -1033,7 +1039,7 @@ are fairly plain.  CALLER_STATE contains the name of the event that
 was being handled when the event was created, or when the event
 watcher that ultimately created the event was registered.
 
-TODO - Rename SENDER_FILE, SENDER_LINE, SENDER_STATE?
+Z<TODO - Rename SENDER_FILE, SENDER_LINE, SENDER_STATE?>
 
 =head3 @_[ARG0..ARG9] or @_[ARG0..$#_]
 
@@ -1082,7 +1088,7 @@ means anything stored there will be available to any other event
 handler regardless of the object.
 
 event_2a is handled by calling C<< $object_2->method_2a(...) >>.  In this
-case C</$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
+case C<$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
 that was passed to the event_1a handler, though.  The methods are resolved
 when the event is handled (late-binding).
 
@@ -1122,7 +1128,7 @@ object upon success, but most applications won't need to save it.
 C<create()> invokes the newly started session's _start event handler
 before returning.
 
-C<create()> also passes the new POE::Session object to L</POE::Kernel>.
+C<create()> also passes the new POE::Session object to L<POE::Kernel>.
 POE's kernel holds onto the object in order to dispatch events to it.
 POE::Kernel will release the object when it detects the object has
 become moribund.  This should cause Perl to destroy the object if
@@ -1131,27 +1137,24 @@ application code has not saved a copy of it.
 C<create()> accepts several named parameters, most of which are optional.
 Note however that the parameters are not part of a hashref.
 
-TODO - Is it time to bring new() back as a synonym for create()?
-TODO PG - NO!  IMHO ->new implies simply creating the object, and 
-TODO that you have to hold onto the object.  ->create implies other actions
-TODO are happening, and that you don't want to hold on to it.
+Z<< Is it time to bring new() back as a synonym for create()?
+PG - NO!  IMHO ->new implies simply creating the object, and
+that you have to hold onto the object.  ->create implies other actions
+are happening, and that you don't want to hold on to it. >>
 
-TODO - Provide forward-compatible "handler" options and methods as
+Z<TODO - Provide forward-compatible "handler" options and methods as
 synonyms for the "state" versions currently supported?
-TODO PG - No, that's for 1.01
+PG - No, that's for 1.01>
 
-TODO - Add a "class_handlers" as a synonym for "package_handlers"?
-TODO PG - Maybe. However, to many synonyms can be a pain for an API.  
+Z<TODO - Add a "class_handlers" as a synonym for "package_handlers"?
+PG - Maybe. However, to many synonyms can be a pain for an API.>
 
-TODO - The above TODOs may be summarized: "deprecate old language"?
-TODO PG - Oh, you are thinking of deprecating the old language... erm... no?
+Z<TODO - The above TODOs may be summarized: "deprecate old language"?
+PG - Oh, you are thinking of deprecating the old language... erm... no?>
 
-
-
-TODO PG - I notice these =head3 are in alphabetical order.  I think
+Z<TODO PG - I notice these =head3 are in alphabetical order.  I think
 TODO all the *_states options should be together.  Followed by heap, args,
-TODO options
-
+TODO options>
 
 =head3 args => ARRAYREF
 
@@ -1523,7 +1526,7 @@ parameters to C<create()>.
 
 Any parameters not recognized by C<create()> must be removed from the
 CREATE_PARAMETERS before C<instantiate()> returns.  C<create()> will
-L<croak|Carp/croak>
+L<croak|Carp>
 if it discovers unknown parameters.
 
 Be sure to return C<$self> from instantiate.
@@ -1630,7 +1633,7 @@ ASSERT_STATES by defaulting the "default" option to true instead of
 false.  See the option() method earlier in this document for details
 about the "default" option.
 
-TODO - It's not much of an assertion if it only warns.
+Z<TODO - It's not much of an assertion if it only warns.>
 
 =head1 SEE ALSO
 
@@ -1677,7 +1680,7 @@ Reduced to its essence:
   my %event_handlers;
   $event_handler{_start} = sub { \%event_handlers };
 
-Note also that a anonymous sub creates a closure on all lexical
+Note also that an anonymous sub creates a closure on all lexical
 variables in the scope it was defined in, even if it doesn't reference
 them.  $session is still being held in a circular reference here:
 
@@ -1719,7 +1722,7 @@ to post the event back to the current session:
     $_[KERNEL]->yield( $next );
   }
 
-Or the L<$_[SESSION]> parameter passed to every event handler, but
+Or the L</$_[SESSION]> parameter passed to every event handler, but
 yield() is more efficient.
 
   next => sub {
@@ -1728,8 +1731,8 @@ yield() is more efficient.
     $_[KERNEL]->post( $_[SESSION], $next );
   }
 
-Along the same lines as L<$_[SESSION]>, a session can respond back to
-the sender of an event by posting to L<$_[SENDER]>.  This is great for
+Along the same lines as L</$_[SESSION]>, a session can respond back to
+the sender of an event by posting to L</$_[SENDER]>.  This is great for
 responding to requests.
 
 If a program must hold onto some kind of dynamic session reference,
