@@ -2,14 +2,14 @@ package POE::Wheel::FollowTail;
 
 use strict;
 
-use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+use vars qw($VERSION @ISA);
+$VERSION = '1.350'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( croak carp );
 use Symbol qw( gensym );
 use POSIX qw(SEEK_SET SEEK_CUR SEEK_END S_ISCHR S_ISBLK);
 use POE qw(Wheel Driver::SysRW Filter::Line);
-use base qw(POE::Wheel);
+push @ISA, qw(POE::Wheel);
 use IO::Handle ();
 use Errno qw(ENOENT);
 
@@ -352,9 +352,13 @@ sub _generate_filehandle_timer {
       if (defined(my $raw_input = $driver->get($$handle))) {
         TRACE_POLL and warn "<poll> " . time . " raw input";
         $filter->get_one_start($raw_input);
-        foreach my $cooked_input (@{$filter->get_one()}) {
-          TRACE_POLL and warn "<poll> " . time . " cooked input";
-          $k->call($ses, $$event_input, $cooked_input, $unique_id);
+        while (1) {
+          my $next_rec = $filter->get_one();
+          last unless @$next_rec;
+          foreach my $cooked_input (@$next_rec) {
+            TRACE_POLL and warn "<poll> " . time . " cooked input";
+            $k->call($ses, $$event_input, $cooked_input, $unique_id);
+          }
         }
 
         # Clear the filehandle's EOF status, if any.
@@ -766,7 +770,7 @@ C<Seek> may also be used with the wheel's tell() method to restore the
 file position after a program restart.  Save the tell() value prior to
 exiting, and load and C<Seek> back to it on subsequent start-up.
 
-TODO - Example.
+Z<TODO - Example.>
 
 =head3 SeekBack
 
@@ -780,7 +784,7 @@ C<Seek> and C<SeekBack> are mutually exclusive.
 See L</Seek> for caveats, techniques, and an explanation of the magic
 that happens when neither C<Seek> nor C<SeekBack> is specified.
 
-TODO - Example.
+Z<TODO - Example.>
 
 =head3 Handle
 
@@ -796,7 +800,7 @@ purview.
 C<Handle> and C<Filename> are mutually exclusive.  One of them is
 required, however.
 
-TODO - Example.
+Z<TODO - Example.>
 
 =head3 Filename
 
