@@ -3,8 +3,7 @@ package POE::Wheel::Run;
 use strict;
 
 use vars qw($VERSION @ISA);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
-@ISA = 'POE::Wheel';
+$VERSION = '1.350'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -13,7 +12,7 @@ use POSIX qw(
 );
 
 use POE qw( Wheel Pipe::TwoWay Pipe::OneWay Driver::SysRW Filter::Line );
-use base qw(POE::Wheel);
+push @ISA, qw(POE::Wheel);
 
 # http://rt.cpan.org/Ticket/Display.html?id=50068
 # Avoid using these constants in Windows' subprocesses (actually
@@ -26,10 +25,24 @@ BEGIN {
 
   local $SIG{'__DIE__'} = 'DEFAULT';
   eval    { require IO::Pty; };
-  if ($@) { eval 'sub PTY_AVAILABLE () { 0 }';  }
+  if ($@) {
+    eval '
+      sub PTY_AVAILABLE () { 0 }
+      sub TIOCSWINSZ_AVAILABLE () { 0 }
+    ';
+  }
   else {
     IO::Pty->import();
     eval 'sub PTY_AVAILABLE () { 1 }';
+
+    eval { require IO::Tty; };
+    if ($@) {
+      eval 'sub TIOCSWINSZ_AVAILABLE () { 0 }';
+    }
+    else {
+      IO::Tty->import('TIOCSWINSZ');
+      eval 'sub TIOCSWINSZ_AVAILABLE () { 1 }';
+    }
   }
 
   if (POE::Kernel::RUNNING_IN_HELL) {
@@ -159,10 +172,18 @@ sub new {
     $conduit = "pipe";
   }
 
-  # TODO - $winsize is not actually used anywhere.  WTF?!
   my $winsize = delete $params{Winsize};
-  croak "Winsize needs to be an array ref"
-    if (defined($winsize) and ref($winsize) ne 'ARRAY');
+
+  if ($winsize) {
+    carp "winsize can only be specified for a Conduit of type pty"
+      if $conduit ne 'pty' and $winsize;
+
+    carp "winsize must be a 4 element arrayref" unless ref($winsize) eq 'ARRAY'
+      and scalar @$winsize == 4;
+
+    carp "winsize only works when IO::Tty::TIOCSWINSZ is"
+      unless TIOCSWINSZ_AVAILABLE;
+  }
 
   my $stdin_event  = delete $params{StdinEvent};
   my $stdout_event = delete $params{StdoutEvent};
@@ -235,6 +256,16 @@ sub new {
     );
   }
 
+  # Did the user mangle stdio?
+  unless (ref($program) eq 'CODE') {
+    croak "Someone has closed or moved STDIN... exec() won't find it"
+      unless defined fileno(STDIN) && fileno(STDIN) == 0;
+    croak "Someone has closed or moved STDOUT... exec() won't find it"
+      unless tied(*STDOUT) || defined fileno(STDOUT) && fileno(STDOUT) == 1;
+    croak "Someone has closed or moved STDERR... exec() won't find it"
+      unless tied(*STDERR) || defined fileno(STDERR) && fileno(STDERR) == 2;
+  }
+
   my (
     $stdin_read, $stdout_write, $stdout_read, $stdin_write,
     $stderr_read, $stderr_write,
@@ -294,7 +325,11 @@ sub new {
 
   # Child.  Parent side continues after this block.
   unless ($pid) {
-    croak "couldn't fork: $!" unless defined $pid;
+    # removed the croak because it wasn't "safe" RT#56417
+    #croak "couldn't fork: $!" unless defined $pid;
+    # ANY OTHER DIE/CROAK/EXIT/WHATEVER in the child MUST use the helper!
+    __PACKAGE__->_warn_and_exit_child( "couldn't fork: $!", int( $! ) )
+      unless defined $pid;
 
     # Stdio should not be tied.  Resolves rt.cpan.org ticket 1648.
     if (tied *STDOUT) {
@@ -320,7 +355,8 @@ sub new {
 
       # Open the slave side of the pty.
       $stdin_read = $stdout_write = $stdin_write->slave();
-      croak "could not create slave pty: $!" unless defined $stdin_read;
+      __PACKAGE__->_warn_and_exit_child( "could not create slave pty: $!", int( $! ) )
+        unless defined $stdin_read;
 
       # For a simple pty conduit, stderr is wedged into stdout.
       $stderr_write = $stdout_write if $conduit eq 'pty';
@@ -329,9 +365,17 @@ sub new {
       # per APITUE 19.4 and 11.10.
       $stdin_read->set_raw();
 
-      # Set the pty conduit (slave side) window size to our window
-      # size.  APITUE 19.4 and 19.5.
-      eval { $stdin_read->clone_winsize_from(\*STDIN) };
+      if (TIOCSWINSZ_AVAILABLE) {
+        if ($winsize) {
+          ioctl($stdin_read, TIOCSWINSZ, pack('vvvv', @$winsize));
+        }
+      }
+      else {
+        # Set the pty conduit (slave side) window size to our window
+        # size.  APITUE 19.4 and 19.5.
+
+        eval { $stdin_read->clone_winsize_from(\*STDIN) } if -T STDIN;
+      }
     }
     else {
       # TODO - Can this be block eval?  Or a do{} block?
@@ -421,14 +465,20 @@ sub new {
         }
       }
 
-      $program->(@$prog_args);
+      # TODO what if the program tries to exit? It needs to use
+      # our _exit_child_any_way_we_can handler...
+      # Should we replace CORE::exit? CORE::die too? blahhhhhh
+      # We've documented that users should not do it, but who knows!
+      eval { $program->(@$prog_args) };
 
-      # Try to force stdio flushing.
-      close STDIN  if defined fileno(STDIN); # Voodoo?
-      close STDOUT if defined fileno(STDOUT);
-      close STDERR if defined fileno(STDERR);
+      my $exitval;
+      if ($@) {
+        chomp $@;
+        warn "$@\n";
+        $exitval = -1;
+      }
 
-      exit __PACKAGE__->_exit_child_any_way_we_can();
+      __PACKAGE__->_exit_child_any_way_we_can( $exitval || 0 );
     }
 
     # Execute an external program.  This gets weird.
@@ -446,12 +496,14 @@ sub new {
     # exec(ARRAY)
     if (ref($program) eq 'ARRAY') {
       exec(@$program, @$prog_args)
-        or die "can't exec (@$program) in child pid $$: $!";
+         or __PACKAGE__->_warn_and_exit_child(
+           "can't exec (@$program) in child pid $$: $!", int( $! ) );
     }
 
     # exec(SCALAR)
     exec(join(" ", $program, @$prog_args))
-      or die "can't exec ($program) in child pid $$: $!";
+      or __PACKAGE__->_warn_and_exit_child(
+        "can't exec ($program) in child pid $$: $!", int( $! ) );
   }
 
   # Parent here.  Close what the parent won't need.
@@ -1093,6 +1145,7 @@ sub kill {
   my ($self, $signal) = @_;
   $signal = 'TERM' unless defined $signal;
   if ( $self->[MSWIN32_GROUP_PID] ) {
+    # TODO use https://rt.cpan.org/Ticket/Display.html?id=67774 when available :)
     Win32::Process::KillProcess( $self->[MSWIN32_GROUP_PID], 293 ) ? 1 : 0;
   }
   else {
@@ -1151,34 +1204,54 @@ sub _redirect_child_stdio_in_hell {
 sub _redirect_child_stdio_sanely {
   my ($class, $stdin_read, $stdout_write, $stderr_write) = @_;
 
+  # Note: we use 2-arg open() below because Perl 5.6 doesn't recognize
+  # the '>&' and '<&' modes with a 3-arg open()
+
   # Redirect STDIN from the read end of the stdin pipe.
   open( STDIN, "<&" . fileno($stdin_read) )
-    or die "can't redirect STDIN in child pid $$: $!";
+    or $class->_warn_and_exit_child(
+      "can't redirect STDIN in child pid $$: $!", int( $! ) );
 
   # Redirect STDOUT to the write end of the stdout pipe.
   open( STDOUT, ">&" . fileno($stdout_write) )
-    or die "can't redirect stdout in child pid $$: $!";
+    or $class->_warn_and_exit_child(
+      "can't redirect stdout in child pid $$: $!", int( $! ) );
 
   # Redirect STDERR to the write end of the stderr pipe.
   open( STDERR, ">&" . fileno($stderr_write) )
-    or die "can't redirect stderr in child: $!";
+    or $class->_warn_and_exit_child(
+      "can't redirect stderr in child pid $$: $!", int( $! ) );
 }
 
 sub _exit_child_any_way_we_can {
   my $class = shift;
+  my $exitval = shift || 0;
+
+  # First make sure stdio are flushed.
+  close STDIN  if defined fileno(STDIN); # Voodoo?
+  close STDOUT if defined fileno(STDOUT);
+  close STDERR if defined fileno(STDERR);
 
   # On Windows, subprocesses run in separate threads.  All the "fancy"
   # methods act on entire processes, so they also exit the parent.
 
   unless (POE::Kernel::RUNNING_IN_HELL) {
     # Try to avoid triggering END blocks and object destructors.
-    eval { POSIX::_exit(0);  };
-    eval { CORE::kill KILL => $$;  };
+    eval { POSIX::_exit( $exitval ); };
+
+    # TODO those methods will not exit with $exitval... what to do?
+    eval { CORE::kill KILL => $$; };
     eval { exec("$^X -e 0"); };
+  } else {
+    eval { CORE::kill( KILL => $$ ); };
+
+    # TODO Interestingly enough, the KILL is not enough to terminate this process...
+    # However, it *is* enough to stop execution of END blocks/etc
+    # So we will end up falling through to the exit( $exitval ) below
   }
 
   # Do what we must.
-  exit(0);
+  exit( $exitval );
 }
 
 # RUNNING_IN_HELL use Win32::Process to create a pucker new shiny
@@ -1190,8 +1263,6 @@ sub _exec_in_hell {
     $program, $prog_args
   ) = @_;
 
-  my $exitcode = 0;
-
   # Close any close-on-exec file descriptors.
   # Except STDIN, STDOUT, and STDERR, of course.
 
@@ -1226,9 +1297,10 @@ sub _exec_in_hell {
   my $w32job;
 
   unless ( $w32job = Win32::Job->new() ) {
-    print $sem_pipe_write "go\n\n";
+    print $sem_pipe_write "go\n\n"; # TODO why the double newline?
     close $sem_pipe_write;
-    die Win32::FormatMessage( Win32::GetLastError() );
+    $class->_warn_and_exit_child(
+      Win32::FormatMessage( Win32::GetLastError() ), Win32::GetLastError() );
   }
 
   my $w32pid;
@@ -1236,21 +1308,31 @@ sub _exec_in_hell {
   unless ( $w32pid = $w32job->spawn( $appname, $cmdline ) ) {
     print $sem_pipe_write "go\n";
     close $sem_pipe_write;
-    die Win32::FormatMessage( Win32::GetLastError() );
+    $class->_warn_and_exit_child(
+      Win32::FormatMessage( Win32::GetLastError() ), Win32::GetLastError() );
   }
 
   print $sem_pipe_write "$w32pid\n";
   close $sem_pipe_write;
 
+  # TODO why 60? Why not MAX_INT so we don't do unnecessary work?
   my $ok = $w32job->watch( sub { 0 }, 60 );
   my $hashref = $w32job->status();
-  $exitcode = $hashref->{$w32pid}->{exitcode};
 
   # In case flushing them wasn't good enough.
   close STDOUT if defined fileno(STDOUT);
   close STDERR if defined fileno(STDERR);
 
-  exit($exitcode);
+  $class->_exit_child_any_way_we_can( $hashref->{$w32pid}->{exitcode} );
+}
+
+# Simple helper to ease the pain of warn+exit
+sub _warn_and_exit_child {
+  my( $class, $warning, $exitval ) = @_;
+
+  warn "$warning\n";
+
+  $class->_exit_child_any_way_we_can( $exitval );
 }
 
 1;
@@ -1432,14 +1514,14 @@ from STDOUT.
 The L<IO::Pty> module is only loaded if "pty" or "pty-pipe" is used.
 It's not a dependency until it's actually needed.
 
-TODO - Example.
+Z<TODO - Example.>
 
 =head4 Winsize
 
 Winsize sets the child process' terminal size.  Its value should be an
-arrayref with two or four elements.  The first two elements must be
-the number of lines and columns for the child's terminal window,
-respectively.  The optional pair of elements describe the terminal's X
+arrayref with four elements.  The first two elements must be the
+number of lines and columns for the child's terminal window,
+respectively.  The second pair of elements describe the terminal's X
 and Y dimensions in pixels:
 
   $_[HEAP]{child} = POE::Wheel::Run->new(
@@ -1867,6 +1949,40 @@ used to tell whether the wheel has more input for the child process.
 
 =head1 TIPS AND TRICKS
 
+=head2 MSWin32 Support
+
+In the past POE::Wheel::Run did not support MSWin32 and users had to
+use custom work-arounds. Then Chris Williams ( BINGOS ) arrived and
+saved the day with his L<POE::Wheel::Run::Win32> module. After some
+testing, it was decided to merge the win32 code into POE::Wheel::Run.
+Everyone was happy!
+
+However, after some investigation Apocalypse ( APOCAL ) found out that
+in some situations it still didn't behave properly. The root cause was
+that the win32 code path in POE::Wheel::Run didn't exit cleanly. This
+means DESTROY and END blocks got executed! After talking with more
+people, the solution was not pretty.
+
+The problem is that there is no equivalent of POSIX::_exit() for MSWin32.
+Hopefully, in a future version of Perl this can be fixed! In the meantime,
+POE::Wheel::Run will use CORE::kill() to terminate the child. However,
+this comes with a caveat: you will leak around 1KB per exec. The code
+has been improved so the chance of this happening has been reduced.
+
+As of now the most reliable way to trigger this is to exec an invalid
+binary. The definition of "invalid binary" depends on different things,
+but what it means is that Win32::Job->spawn() failed to run. This will
+force POE::Wheel::Run to use the workaround to exit the child. If this
+happens, a very big warning will be printed to the STDERR of the child
+and the parent process will receive it.
+
+If you are a Perl MSWin32 hacker, PLEASE help us with this situation! Go
+read rt.cpan.org bug #56417 and talk with us/p5p to see where you can
+contribute.
+
+Thanks again for your patience as we continue to improve POE::Wheel::Run
+on MSWin32!
+
 =head2 Execution Environment
 
 It's common to scrub a child process' environment, so that only
